package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.cache.CommonCacheService;
import com.qmth.sop.business.entity.BasicAttachment;
import com.qmth.sop.business.entity.SysConfig;
import com.qmth.sop.business.mapper.BasicAttachmentMapper;
import com.qmth.sop.business.service.BasicAttachmentService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.ExceptionResultEnum;
import org.apache.commons.io.FilenameUtils;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * <p>
 * 所有附件记录表 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2023-07-17
 */
@Service
public class BasicAttachmentServiceImpl extends ServiceImpl<BasicAttachmentMapper, BasicAttachment> implements BasicAttachmentService {

    @Resource
    CommonCacheService commonCacheService;

    /**
     * 验证附件
     *
     * @param file
     */
    @Override
    public void validateAttachment(MultipartFile file) {
        String fileName = FilenameUtils.getBaseName(file.getOriginalFilename());
        String format = "." + FilenameUtils.getExtension(file.getOriginalFilename());

        SysConfig sysConfig = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_TYPE);
        Optional.ofNullable(sysConfig).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件类型"));
        List<String> attachmentTypeList = Arrays.asList(sysConfig.getConfigValue().replaceAll("\\[", "").replaceAll("\\]", "").split(","));
        if (Objects.nonNull(format)) {
            long count = attachmentTypeList.stream().filter(s -> format.equalsIgnoreCase(s)).count();
            if (count == 0) {
                throw ExceptionResultEnum.ERROR.exception("文件格式只能为" + attachmentTypeList.toString());
            }
        }

        SysConfig sysConfigLength = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_LENGTH);
        Optional.ofNullable(sysConfigLength).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件名称长度"));
        int attachmentLength = Integer.parseInt(sysConfigLength.getConfigValue());
        if (Objects.nonNull(fileName) && fileName.length() > attachmentLength) {
            throw ExceptionResultEnum.ERROR.exception("文件名长度不能超过" + attachmentLength + "个字符");
        }
        long size = file.getSize();
        BigDecimal b = new BigDecimal(size);
        BigDecimal num = new BigDecimal(1024);
        b = b.divide(num, 2, BigDecimal.ROUND_HALF_UP).divide(num, 2, BigDecimal.ROUND_HALF_UP)
                .setScale(2, BigDecimal.ROUND_HALF_UP);

        SysConfig sysConfigSize = commonCacheService.addSysConfigCache(SystemConstant.ATTACHMENT_SIZE);
        Optional.ofNullable(sysConfigSize).orElseThrow(() -> ExceptionResultEnum.ERROR.exception("未配置附件大小"));
        double attachmentSize = Double.valueOf(sysConfigSize.getConfigValue());
        if (b.doubleValue() > attachmentSize) {
            throw ExceptionResultEnum.ERROR.exception("文件大小不能超过" + attachmentSize + "MB");
        }
    }
}
